
/* 
	dynprof.h - Header file for dynamic profiling code

ADDED_AD : 02/12/2004: First version before cvs checkin. This is a more optimized, cleaner source file than 
						used previously. Functionality for dynamic profiling only contained here now.
*/
#include "memsimlib.h"
#include "varray.h"

typedef int myint;

#define  ROWWISE 1
#define  SCATTERGATHER 2
#define  NONE 3 

#define PROF_MAXLINE  256           /*max size of a line (string)*/
#define PROF_MEMCFGFILE "mem.cfg"   /*name of the file used to store mem config info*/
#define STAMP_DUMP	"out.stamp"		/* name of file used to hold timestamp info */
#define PROF_MAXNREGS 64           /*max num of regs for compiler analysis*/
#define PROF_CALLGRAPHOUT "out.callgraph" //initial callgraph file
#define PROF_DPRGOUT "out.dprg" //initial callgraph file
#define PROF_GLOBALS "out.globals" //dumps out all global vars seen






 
struct basic_block_info{
   varray_type myrefs;
   int num;
   int id;
   int loop_level;
   int depth;
   int highest_depth;
   
   
};

typedef struct basic_block_info bblock_info;


struct partializing_info{
int dimension;
int type;  
varray_type myrefs;
char base_array_name[10];
int no_of_dimensions;
int dimension_sizes;
int starting_offset;
int stride;
};





//structure used to store variable info
typedef struct {
	char name[256];	//name of variable
    int nread;     //total number of reads
    int nwrite;    //total number of writes
    int size;      //size of the variable (in bytes)
    int ispublic;  //0 = var is static (not public), 1 = var is public
	int index;	//stores index of variable
	//char type[256]; //stores type of variable(int,char,etc) maybe used later
	//int sram;		//shows if it should be allocated to sram
} profVars_t;

/****************************ADDED_SK****************************/
#define PROF_DEBUG "out.debug" //initial callgraph file
#define MAX_NODES 2 
// Following define the type of Dprg nodes possible
#define B_BLOCK -1 
// Following define the type of Dprg nodes possible
/*#define PROC 1
#define LOOP 2
#define VAR 3
#define LOOP_HEADER 4
#define IF_THEN 5
#define B_BLOCK 6
*/
#define INPUT_DPRG "dprg.input"
#define  OUT_STACK "out.stack"
#define MAXNODES  100;

typedef struct{
	char name[128];
	int id;
	int fileid;
	int functionid;
	int markerid;
	int type;
	int size;
	int loop_index;
	char scope[32];  
	int path_id;

	int no_of_children;
        int  parent_bb;
	int  current_bb;
	int end_bb;
	int start_bb;
	dll_t* childlist;
	
}dynprofNodeInfo_t;



typedef struct{
	int id;
	int type;
	int read_weight;
	int write_weight;
	int invocations;  // a running counter used for different purposes like invocations to a region, counter to the next use in history
	int no_of_children;
	int no_of_parents;
	int no_of_paths;
	int size;
	 int allocation_point;   // is either 1/0 in the dprg for non variables nodes, undefined for variables nodes
				 //, but during allocation stores the offset in the history table for variable nodes.
	//  allca allocation[1]; // for a region stores allocation information at different time stamps contained in it.

	//char parents[5][50];
	// int* weight_of_path;
	//t_stamp* time_stamps_for_edge;
	char scope[32];  
	int allocation_choosen;
	dll_t* child_list;
	
}dynprofAllocNode_t;

/***************************END_SK*****************************/

/******** Used to generate DYNAMIC profiling information ********/
    /*Inserts the initialization function calls necessary for generating dynamic profiling information. */
int dynprofInsertMarkers(tree decl, rtx insns,int FileID,int FuncID);
	/* Isnerts markers, according to slightly different loop semantics */
//int dynprofInsertDprgMarkers(tree decl, rtx insns,int FileID,int FuncID);
	/* Inserts me style region markers */
int dynprofInsertRegionMarkers(tree decl, rtx insns,int FileID,int FuncID);
int dynprofInsertDprgMarkers(tree decl, rtx insns,int FileID,int FuncID,int markerid);
	/*inserts hooks to avoid library call profiling */
int dynprofInsertProfCalls(tree decl, rtx insns);
int dynprofInsertProfCallsDprg(tree decl, rtx insns,int a, int b,int c);//this is the version for 2ndary ifthen call search
    /*function used to initialize memory profiling */
int dynprofInit(char *basefilename);
    /*clean up */
int dynprofDestroy(void);
//BELOW is for ad region profiling
/* Dumps callgraph and static dprg */
int dynprofDumpStaticInfo(tree decl, rtx insns,int fileid,int funcid,int FuncMark);
	/* Dumps static dprg and inserts initial markers for loops and ifthen conditional blocks */

int dynprofDumpStaticGraph(tree decl, rtx insns);
int dynprofInsertChildNode(int type);
int dynprofPrintGraph(void);
int dynprofInsertIfThenStartAD(rtx tmprtx,int MarkerID, int Type);
int	dynprofInsertIfThenEndAD(rtx tmprtx,int MarkerID, int Type);
int dynprofInsertLoopStartAD(rtx tmprtx,int MarkerID, int Type);
int	dynprofInsertLoopEndAD(rtx tmprtx,int MarkerID, int Type);
int	dynprofInsertLoopContAD(rtx tmprtx,int MarkerID, int Type);
int get_mem_exprAD(tree expr);
int FindVarAD(rtx rtxnode);
int dynprofDumpTree(tree decl);
//end ad region profiling

//Used to dump out callgraph for ilp profiling
int dynprofDumpStaticILP(tree decl, rtx insns);
int dynprofInsertSaveRegs(rtx tmprtx);
int dynprofInsertRestoreRegs(rtx tmprtx);
int dynprofCheckCALL(rtx rtxnode, char *funcname);
int dynprofClearList(dll_t *list);//cleanup code
